package cz.cvut.fel.alg;

/**
 * Created by karel on 3/21/2016.
 */
public class BinarySearchTree extends Tree<BinarySearchTree> {

    private BinarySearchTree parent;
    
    public BinarySearchTree(int value) {
        this(value, null);
    }
    
    public BinarySearchTree(int value, BinarySearchTree parent) {
        super(value);
        this.parent = parent;
    }

    public BinarySearchTree getParent() {
        return parent;
    }

    public void setParent(BinarySearchTree parent) {
        this.parent = parent;
    }

    /**
     * This method finds minimal value that is stored in the tree.
     */
    @Override
    public int minValue() {
        BinarySearchTree node = this;
        while(node.getLeftSubTree() != null)
            node = node.getLeftSubTree();
        return node.getValue();
    }

    public BinarySearchTree findClosest(int key) {
        if(getValue() < key && getRightSubTree() != null) {
            return getRightSubTree().findClosest(key);
        } else if(getValue() > key && getLeftSubTree() != null) {
            return getLeftSubTree().findClosest(key);
        } else return this;
    }

    public BinarySearchTree find(int key) {
        BinarySearchTree closest = findClosest(key);
        if(closest.getValue() == key) {
            return closest;
        } else {
            return null;
        }
    }

    public boolean constains(int key) {
        return find(key) != null;
    }

    public void insert(int key) {
        BinarySearchTree closest = findClosest(key);
        if(closest.getValue() == key) throw new RuntimeException(String.format("Key {%d} is already present in the BST", key));
        else if(closest.getValue() > key) {
            closest.setLeftSubTree(new BinarySearchTree(key, closest));
        } else {
            closest.setRightSubTree(new BinarySearchTree(key, closest));
        }
    }

    /**
     * This method deletes {@code key} from the tree.
     * @param key The value to be deleted from this BST.
     */
    public void delete(int key) {
        BinarySearchTree closest = find(key);
        // we want to delete key that is not in the tree, do nothing
        if(closest == null)
            return;

        closest.delete();
    }

    
    private void delete() {
        // if this is a leaf, delete link to it from parent
        if (this.getLeftSubTree() == null && this.getRightSubTree() == null) {
            if (getParent() != null && this == getParent().getLeftSubTree())
                getParent().setLeftSubTree(null);
            else if (getParent() != null && this == getParent().getRightSubTree())
                getParent().setRightSubTree(null);
        } // else if there is no left subtree, use the right subtree value
        else if (this.getLeftSubTree() == null) {
            BinarySearchTree lowest = this.getRightSubTree();
            while (lowest.getLeftSubTree()!= null)
                lowest = lowest.getLeftSubTree();
            this.value = lowest.getValue();
            lowest.delete();
        } // otherwise find the greatest element in the left subtree
        else {
            BinarySearchTree greatest = this.getLeftSubTree();
            while (greatest.getRightSubTree() != null)
                greatest = greatest.getRightSubTree();
            this.value = greatest.getValue();
            greatest.delete();
        }
    }
}
