package cz.cvut.fel.alg;

import java.util.Arrays;
import java.util.Random;
import static org.hamcrest.CoreMatchers.equalTo;
import static org.hamcrest.CoreMatchers.is;
import org.junit.Test;
import static org.junit.Assert.*;
import org.junit.FixMethodOrder;
import org.junit.runners.MethodSorters;

/**
 * Test class for binary search tree.
 *
 * @author Petr Ryšavý, Karel Horák
 */
@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class BinarySearchTest {
    
    public BinarySearchTest() {
    }

    @Test
    public void test00_BinarySearchExisting() {
        int[] array = new int[] {1, 7, 10, 15, 21, 42, 48, 90, 101, 107, 111, 125, 173, 211};
        assertThat(BinarySearch.binarySearch(array, 42), is(equalTo(5)));
    }

    @Test
    public void test01_BinarySearchMissing() {
        int[] array = new int[] {1, 7, 10, 15, 21, 42, 48, 90, 101, 107, 111, 125, 173, 211};
        assertThat(BinarySearch.binarySearch(array, 43), is(equalTo(-1)));
    }

    @Test
    public void test02_BinarySearchEmpty() {
        int[] array = new int[] {};
        assertThat(BinarySearch.binarySearch(array, 42), is(equalTo(-1)));
    }

    @Test
    public void test03_BinarySearchSingleElement() {
        int[] array = new int[] {42};
        assertThat(BinarySearch.binarySearch(array, 42), is(equalTo(0)));
    }
    
    @Test
    public void test04_BinarySearchFirstElement() {
        int[] array = new int[] {1, 7, 10, 15, 21, 42, 48, 90, 101, 107, 111, 125, 173, 211};
        assertThat(BinarySearch.binarySearch(array, 1), is(equalTo(0)));
    }
    
    @Test
    public void test05_BinarySearchLastElement() {
        int[] array = new int[] {1, 7, 10, 15, 21, 42, 48, 90, 101, 107, 111, 125, 173, 211};
        assertThat(BinarySearch.binarySearch(array, 211), is(equalTo(array.length - 1)));
    }
    
    @Test
    public void test06_BinarySearchMedian() {
        int[] array = new int[] {1, 7, 10, 15, 21};
        assertThat(BinarySearch.binarySearch(array, 10), is(equalTo(2)));
    }
    
    @Test
    public void test07_BinarySearchRandomSmall() {
        Random r = new Random();
        int[] array = new int[100];
        for(int i = 0; i < array.length; i++)
            array[i] = r.nextInt(array.length * 2);
        Arrays.sort(array);
        final int query = r.nextInt(array.length);
        final int java = Arrays.binarySearch(array, query);
        assertThat(BinarySearch.binarySearch(array, query), is(equalTo(java < 0 ? -1 : java)));
    }
    
    @Test
    public void test08_BinarySearchRandomBig() {
        Random r = new Random();
        int[] array = new int[10000];
        for(int i = 0; i < array.length; i++)
            array[i] = r.nextInt(array.length * 2);
        Arrays.sort(array);
        final int query = r.nextInt(array.length);
        final int java = Arrays.binarySearch(array, query);
        assertThat(BinarySearch.binarySearch(array, query), is(equalTo(java < 0 ? -1 : java)));
    }
}
