package cz.cvut.fel.alg;

import static org.hamcrest.CoreMatchers.is;
import org.junit.Assert;
import org.junit.Before;
import org.junit.FixMethodOrder;
import org.junit.Test;
import org.junit.runners.MethodSorters;

/**
 * Test of the {@code isAVL} method.
 *
 * @author Petr Ryšavý, Karel Horák
 */
@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class IsAVLTest {

    private Tree tree;

    @Before
    public void init() {
        tree = new Tree(42,
                new Tree(33,
                        new Tree(15, new Tree(7), null, 1),
                        new Tree(39, new Tree(35), new Tree(41), 1), 2),
                new Tree(59,
                        new Tree(47),
                        new Tree(105,
                                new Tree(70),
                                null, 1), 2), 3);
    }

    @Test
    public void test01_isAVLTrue() {
        Assert.assertThat(tree.isAVL(), is(true));
    }

    @Test
    public void test02_isAVLTrue() {
        tree = new Tree(51,
                new Tree(34,
                        new Tree(13, new Tree(8), new Tree(22), 1),
                        new Tree(40, null, new Tree(45), 1), 2),
                new Tree(76, new Tree(68), new Tree(92), 1),
                3);
        Assert.assertThat(tree.isAVL(), is(true));
    }

    @Test
    public void test11_isBSTFalse() {
        // left and right are swithched
        tree = new Tree(42,
                new Tree(33,
                        new Tree(39, new Tree(7), null, 1),
                        new Tree(15, new Tree(35), new Tree(41), 1), 2),
                new Tree(59,
                        new Tree(47),
                        new Tree(105,
                                new Tree(70),
                                null, 1), 2), 3);

        Assert.assertThat(tree.isAVL(), is(false));
    }

    @Test
    public void test12_isBSTFalse() {
        // the problem happens in the second level 59 has right child 105 with left 30
        tree = new Tree(42,
                new Tree(33,
                        new Tree(15, new Tree(7), null, 1),
                        new Tree(39, new Tree(35), new Tree(41), 1), 2),
                new Tree(59,
                        new Tree(47),
                        new Tree(105,
                                new Tree(30),
                                null, 1), 2), 3);

        Assert.assertThat(tree.isAVL(), is(false));
    }

    @Test
    public void test21_isDepthProperlySetFalse() {
        tree.height = 4;
        Assert.assertThat(tree.isAVL(), is(false));
    }

    @Test
    public void test22_isDepthProperlySetFalse() {
        tree.getLeftSubTree().getLeftSubTree().getLeftSubTree().height = -1;
        Assert.assertThat(tree.isAVL(), is(false));
    }

    @Test
    public void test31_isBalancedFalse() {
        // root is broken
        tree = new Tree(42,
                new Tree(33,
                        new Tree(15, new Tree(7), null, 1),
                        new Tree(39, new Tree(35), new Tree(41), 1), 2),
                new Tree(59,
                        null,
                        new Tree(105,
                                new Tree(70),
                                null, 1), 2), 3);
        Assert.assertThat(tree.isAVL(), is(false));
    }

    @Test
    public void test32_isBalancedFalse() {
        // root is ok, but one of its child is broken
        tree = new Tree(42,
                new Tree(33),
                new Tree(59,
                        new Tree(47),
                        new Tree(105,
                                new Tree(70),
                                null, 1), 2), 3);
        Assert.assertThat(tree.isAVL(), is(false));
    }

}
