package cz.cvut.fel.alg;

import org.junit.Before;
import org.junit.FixMethodOrder;
import org.junit.Test;
import org.junit.runners.MethodSorters;

import java.util.*;

import static org.hamcrest.CoreMatchers.equalTo;
import static org.hamcrest.CoreMatchers.is;
import static org.hamcrest.Matchers.contains;
import static org.junit.Assert.assertThat;

/**
 *
 * @author Karel Horák
 */
@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class SimpleBSTTest {

    private BinarySearchTree tree;
    private Random random;

    @Before
    public void buildSimpleTree() {
        Tree _tree = new Tree(42,
                new Tree(33,
                        new Tree(15, new Tree(7), null),
                        new Tree(39, new Tree(35,null,null), new Tree(41,null,null))),
                new Tree(59,
                        new Tree(47, null, null),
                        new Tree(105,
                                new Tree(70,null,null),
                                null)));
        tree = makeBST(_tree);

        random = new Random();
    }

    private BinarySearchTree makeBST(Tree t) {
        if(t == null) return null;

        BinarySearchTree bst = new BinarySearchTree(t.getValue());
        bst.setLeftSubTree(makeBST(t.getLeftSubTree()));
        if(bst.getLeftSubTree() != null)
            bst.getLeftSubTree().setParent(bst);
        bst.setRightSubTree(makeBST(t.getRightSubTree()));
        if(bst.getRightSubTree() != null)
            bst.getRightSubTree().setParent(bst);
        return bst;
    }

    private ArrayList<Integer> inorder(BinarySearchTree bst) {
        ArrayList<Integer> traversal = new ArrayList<>();
        bst.walkInorder(traversal);
        return traversal;
    }

    @Test
    public void test1_insert() {
        for(int i = 0 ; i < 1000 ; i++) {
            ArrayList<Integer> before = inorder(tree);
            int insert = random.nextInt(5000);

            if(tree.constains(insert)) continue;
            else {
                tree.insert(insert);
                before.add(insert);
                Collections.sort(before);

                assertThat(inorder(tree), contains(before.toArray(new Integer[0])));
            }
        }
    }

    @Test
    public void test2_MinValue() {
        test1_insert();

        ArrayList<Integer> data = inorder(tree);
        assertThat(tree.minValue(), is(equalTo(data.get(0))));
    }

    @Test
    public void test3_Delete() {
        test1_insert();

        ArrayList<Integer> data = inorder(tree);
        Collections.shuffle(data);

        for(int i = 0 ; i < data.size()-1 ; i++) {
            tree.delete(data.get(i));
            ArrayList<Integer> copied = new ArrayList<>(data.size()-i-1);
            for(int j = i+1 ; j < data.size() ; j++) {
                copied.add(data.get(j));
            }
            Collections.sort(copied);

            assertThat(inorder(tree), contains(copied.toArray(new Integer[0])));
        }
    }
}
